﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Research.Kinect.Nui;

namespace Sensor.Gesture.Common
{
    /// <summary>
    /// All return values in parameter space unless stated otherwise.
    /// </summary>
    public static class JointUtil
    {
        public static Point Position(Joint joint)
        {
            float depth_x, depth_y;
            Main.NUI.SkeletonEngine.SkeletonToDepthImage(joint.Position, out depth_x, out depth_y); // [0.0 1.0]
            return new Point(depth_x, depth_y);
        }

        public static int DepthMillimeter(Joint joint)
        {
            float depth_x, depth_y;
            short depth_z;
            Main.NUI.SkeletonEngine.SkeletonToDepthImage(joint.Position, out depth_x, out depth_y, out depth_z); // [0.0 1.0]
            return depth_z;
        }

        public static int LeftHandDepth(JointsCollection js)
        {
            return DepthMillimeter(js[JointID.HandLeft]);
        }

        public static int ShoulderCenterDepth(JointsCollection js)
        {
            return DepthMillimeter(js[JointID.ShoulderCenter]);
        }

        public static Point LeftHand(JointsCollection js)
        {
            return JointUtil.Position(js[JointID.HandLeft]);
        }

        public static Point LeftWrist(JointsCollection js)
        {
            return JointUtil.Position(js[JointID.WristLeft]);
        }

        public static Point RightHand(JointsCollection js)
        {
            return JointUtil.Position(js[JointID.HandRight]);
        }

        public static Point RightWrist(JointsCollection js)
        {
            return JointUtil.Position(js[JointID.WristRight]);
        }

        public static Point ShoulderCenter(JointsCollection js)
        {
            return JointUtil.Position(js[JointID.ShoulderCenter]);
        }

        public static Point JointX(JointsCollection js, JointID joint)
        {
            return JointUtil.Position(js[joint]);
        }
    }
}
