﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Research.Kinect.Nui;
using sensor;
using Sensor.Gesture.Transform;
using Sensor.MouseAction;
using Sensor.Gesture.GestureHandler;

namespace Sensor.Gesture
{
    public class GestureDispatcher : IGestureDispatcher
    {
        private IGestureHandler m_current_handler00;
        private ITransform m_transform;
        private IStatus m_status;

        IGestureHandler m_no_action_gesture;
        private int m_current_tracking_id = -1;

        public GestureDispatcher(IStatus status)
        {
            m_status = status;
            m_transform = new Transform.Transform();
        }

        public void Start()
        {
            Main.NUI.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(EventSkeletonFrameReady);
            m_no_action_gesture = new NoActionGesture(m_status);
        }

        public void EventSkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            m_status.SetStatus3("Gesture handler got event");

            SkeletonData skeleton =
                (from s in e.SkeletonFrame.Skeletons
                 where s.TrackingState == SkeletonTrackingState.Tracked
                 orderby s.TrackingID == m_current_tracking_id descending
                 select s
                ).ToArray<SkeletonData>().FirstOrDefault();
            if (skeleton == null) return;
            m_current_tracking_id = skeleton.TrackingID;
            JointsCollection joints = skeleton.Joints;

            if (m_current_handler00 == null)
            {
                m_current_handler00 = DecideHandler00(joints);
            }

            if (m_current_handler00 == null)
            {
                m_status.SetStatus3("GestureDispatcher. No handlers found.");
                return;
            }

            m_status.SetStatus3("GestureDispatcher. Found handler.");
            bool continue_handling = m_current_handler00.HandleMoveEvent(joints);
            if (!continue_handling)
            {
                m_current_handler00 = null;
            }
        }

        private IGestureHandler DecideHandler00(JointsCollection js)
        {
            // We first give the activate gesture handler a chance to swallow events,
            // which should happen when the hand is not in the control area
            if (m_no_action_gesture.CanHandleEvent(js))
            {
                return m_no_action_gesture;
            }

            // go from specific gestures to general ones

            IGestureHandler any_two_hands_gesture = new TwoHandsGesture(m_transform, m_status);
            if (any_two_hands_gesture.CanHandleEvent(js))
            {
                return any_two_hands_gesture;
            }

            IGestureHandler left_down = new OneHandStretchedAndFingersPointingDownGesture(m_status, new ClickAction(MouseButton.Left), JointID.HandLeft);
            if (left_down.CanHandleEvent(js))
            {
                return left_down;
            }

            IGestureHandler right_down = new OneHandStretchedAndFingersPointingDownGesture(m_status, new ClickAction(MouseButton.Right), JointID.HandRight);
            if (right_down.CanHandleEvent(js))
            {
                return right_down;
            }

            IGestureHandler left_hand_movement_gesture = new OneHandMovementGesture(m_status, JointID.HandLeft, new MouseMoveAction(m_transform, 40));
            if (left_hand_movement_gesture.CanHandleEvent(js))
            {
                return left_hand_movement_gesture;
            }

            IGestureHandler right_hand_movement_gesture = 
                new OneHandMovementGesture(
                    m_status,
                    JointID.HandRight,
                    new MouseDragAction(m_transform, MouseButton.Right)
                );
            if (right_hand_movement_gesture.CanHandleEvent(js))
            {
                return right_hand_movement_gesture;
            }

            return null;
        }
    }
}
