﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;
using sensor;
using Sensor.MouseAction;
using Sensor.Common;

namespace Sensor.Gesture.GestureHandler
{
    public class NoActionGesture : IGestureHandler
    {
        // we want either or two hands to be close enough to shoulder center
        private const double s_activation_radius_from_shoulder_ps = 0.1;

        private IStatus m_status;

        public NoActionGesture(IStatus status)
        {
            m_status = status;
        }

        public bool HandleMoveEvent(JointsCollection js)
        {
            if (CanHandleEvent(js))
            {
                // We simply swallow events as long as we think there is no gesture.
                return true;
            }
            return false;
        }

        public bool CanHandleEvent(JointsCollection js)
        {
            if (!CommonGestureDecisions.StretchedOutFully(js, JointID.HandLeft) &&
                !CommonGestureDecisions.StretchedOutFully(js, JointID.HandRight))
            {
                m_status.SetStatus2(
                    "NoActionGesture. No stretched your hands, jump, or do something." + 
                    " Left = " + JointUtil.LeftHandDepth(js) + " " + JointUtil.ShoulderCenterDepth(js));
                return true;
            }

            m_status.SetStatus2("NoActionGesture. Some gesture found. No more swallowing movements.");
            return false;
        }
    }
}
