﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using EventGeneration;
using Sensor.Gesture.Common;
using Microsoft.Research.Kinect.Nui;
using sensor;
using Sensor.Gesture.Transform;
using Sensor.MouseAction;
using Sensor.Common;

namespace Sensor.Gesture.GestureHandler
{
    public class OneHandMovementGesture : IGestureHandler
    {
        private IStatus m_status;
        private IMouseMoveAction m_mouse_move_action;
        private JointID m_hand;

        public OneHandMovementGesture(
            IStatus status,
            JointID joint,
            IMouseMoveAction movement_action
        )
        {
            m_status = status;
            m_mouse_move_action = movement_action;
            m_hand = joint;
        }

        public bool HandleMoveEvent(JointsCollection js){
            if (!KeepHandlingEvent(js)
            ){
                return false;
            }

            m_mouse_move_action.MoveBy(JointLocation(js));
            m_status.SetStatus1("MouseMoveGesture. Moving.");
            return true;
        }

        private bool KeepHandlingEvent(JointsCollection js)
        {
            Point joint_location = JointLocation(js);
            if (PointUtil.Magnitude(joint_location) == 0)
            {
                m_status.SetStatus1("MouseMoveGesture. New position is zero. Can't handle.");
                m_mouse_move_action.Finish();
                return false;
            }

            if (!CommonGestureDecisions.StretchedOutPartially(js, m_hand))
            {
                m_status.SetStatus1("MouseMoveGesture. Hand not stretched out at least partially.");
                m_mouse_move_action.Finish();
                return false;
            }

            return true;
        }

        public bool CanHandleEvent(JointsCollection js){
            Point joint_location = JointLocation(js);
            if (PointUtil.Magnitude(joint_location) == 0)
            {
                m_status.SetStatus1("MouseMoveGesture. New position is zero. Can't handle.");
                m_mouse_move_action.Finish();
                return false;
            }

            if (!CommonGestureDecisions.StretchedOutFully(js, m_hand))
            {
                m_status.SetStatus2("MouseMoveGesture. Hand not stretched out far enough.");
                m_mouse_move_action.Finish();
                return false;
            }

            return true;
        }

        private Point JointLocation(JointsCollection js)
        {
            switch (m_hand)
            {
                case JointID.HandLeft:
                    return JointUtil.LeftHand(js);
                case JointID.HandRight:
                    return JointUtil.RightHand(js);
                default:
                    throw new ArgumentException("Unexpected joint : " + m_hand);
            }
        }
    }
}
