﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;
using sensor;
using EventGeneration;
using Sensor.MouseAction;
using Sensor.Common;

namespace Sensor.Gesture.GestureHandler
{
    public class OneHandStretchedAndFingersPointingDownGesture : IGestureHandler
    {
        private const double s_maximum_wrist_to_hand_distance = 0.05;
        private const int s_minimum_second_between_clicks = 5;

        private IStatus m_status;
        private INoArgumentAction m_action;
        private JointID m_hand;

        public OneHandStretchedAndFingersPointingDownGesture(IStatus status, INoArgumentAction action, JointID hand)
        {
            m_status = status;
            m_action = action;
            m_hand = hand;
        }

        public bool HandleMoveEvent(JointsCollection js)
        {
            if (!CanHandleEvent(js)) return false;
            m_action.Execute();
            return true;
        }

        public bool CanHandleEvent(JointsCollection js)
        {
            if (!CommonGestureDecisions.StretchedOutFully(js, m_hand))
            {
                m_status.SetStatus2("OneHandDownGesture. Hand not stretched out far enough.");
                return false;
            }

            double wrist_y_to_hand_y_distance = JointLocation(js).Y - WristLocation(js).Y;
            if (wrist_y_to_hand_y_distance < -0.03) return false; // palm help up

            m_status.SetStatus1("OneHandDownGesture. Handling.");
            return true;
        }

        private Point JointLocation(JointsCollection js)
        {
            switch (m_hand)
            {
                case JointID.HandLeft:
                    return JointUtil.LeftHand(js);
                case JointID.HandRight:
                    return JointUtil.RightHand(js);
                default:
                    throw new ArgumentException("Unexpected joint : " + m_hand);
            }
        }

        private Point WristLocation(JointsCollection js)
        {
            switch (m_hand)
            {
                case JointID.HandLeft:
                    return JointUtil.LeftWrist(js);
                case JointID.HandRight:
                    return JointUtil.RightWrist(js);
                default:
                    throw new ArgumentException("Unexpected joint : " + m_hand);
            }
        }
    }
}
