﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;
using System.Windows;
using sensor;
using Sensor.Gesture.Transform;
using Sensor.MouseAction;
using Sensor.Common;
using Sensor.Gesture.GestureHandler;

namespace Sensor.Gesture
{
    public class TwoHandsGesture : IGestureHandler
    {
        private const double s_zoom_decision_threshold = 0.05;
        private const double s_minimum_seconds_before_decision = 1;
 
        private ITransform m_transform;
        private Nullable<double> m_previous_distance_from_left_to_right;
        private DateTime m_time_previous_recorded;
        private IGestureHandler m_concrete_handler00;
        private IStatus m_status;

        public TwoHandsGesture(ITransform transform, IStatus status)
        {
            m_transform = transform;
            m_concrete_handler00 = null;
            m_status = status;
        }

        public bool HandleMoveEvent(JointsCollection js)
        {
            if (m_concrete_handler00 != null){
                return m_concrete_handler00.HandleMoveEvent(js);
            }

            if (!m_previous_distance_from_left_to_right.HasValue)
            {
                m_previous_distance_from_left_to_right = 
                    PointUtil.Distance(JointUtil.LeftHand(js), JointUtil.RightHand(js));
                m_time_previous_recorded = DateTime.Now;
                return true;
            }

            if ((DateTime.Now - m_time_previous_recorded).Seconds < s_minimum_seconds_before_decision)
            {
                m_status.SetStatus1("TwoHandsGesture. Move your hands together or away from each other.");
                return true;
            }

            double current_distance_from_left_to_right =
                PointUtil.Distance(JointUtil.LeftHand(js), JointUtil.RightHand(js));
            double difference = m_previous_distance_from_left_to_right.Value - current_distance_from_left_to_right;

            if (Math.Abs(difference) < 0.005)
            {
                m_status.SetStatus1("TwoHandsGesture. Movement not sufficient yet.");
                return true;
            }

            if (Math.Abs(difference) > s_zoom_decision_threshold)
            {
                m_status.SetStatus1("TwoHandsGesture. Hands moving away or closer.");
                m_concrete_handler00 = new TwoHandsMoveAwayOrCloserGesture(m_transform, m_status);
            }
            else
            {
                m_status.SetStatus1("TwoHandGesture. Hands moving together.");
                m_concrete_handler00 = new TwoHandsMoveTogetherGesture(m_transform, m_status);
            }
            return true;
        }

        public bool CanHandleEvent(JointsCollection js)
        {
            bool can_handle =
                   CommonGestureDecisions.StretchedOutPartially(js, JointID.HandLeft) &&
                   CommonGestureDecisions.StretchedOutPartially(js, JointID.HandRight);
            if (!can_handle) m_previous_distance_from_left_to_right = null;
            return can_handle;
        }
    }
}
