﻿using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;
using sensor;
using Sensor.Gesture.Transform;
using Sensor.MouseAction;
using Sensor.Common;

namespace Sensor.Gesture.GestureHandler
{
    public class TwoHandsMoveAwayOrCloserGesture : IGestureHandler
    {
        private ITransform m_transform;
        private IStatus m_status;
        private ISmoothedTwoPointMouseMovementAction m_mouse_move_action;
        private bool m_first_time;

        public TwoHandsMoveAwayOrCloserGesture(ITransform transform, IStatus status)
        {
            m_transform = transform;
            m_status = status;
            m_mouse_move_action = new TwoPointMouseMoveAction(transform, status);
            m_first_time = true;
        }

        public bool HandleMoveEvent(JointsCollection js)
        {
            if (!KeepHandlingEvent(js))
            {
                EventGeneration.MouseEvent.EventEndLeftDrag();
                EventGeneration.KeyboardEvent.EventKeyUpShift();
                return false;
            }

            if (m_first_time)
            {
                EventGeneration.KeyboardEvent.EventKeyDownShift();
                EventGeneration.MouseEvent.EventStartLeftDrag();
                m_first_time = false;
            }

            m_status.SetStatus1("TwoHandsMoveAwayOrCloseGesture");
            m_mouse_move_action.MoveTo(JointUtil.LeftHand(js), JointUtil.RightHand(js));
            return true;
        }

        public bool KeepHandlingEvent(JointsCollection js)
        {
            bool keep_handling =
                CommonGestureDecisions.StretchedOutPartially(js, JointID.HandLeft) ||
                CommonGestureDecisions.StretchedOutPartially(js, JointID.HandRight);
            if (!keep_handling)
            {
                m_first_time = true;
            }
            return keep_handling;
        }

        public bool CanHandleEvent(JointsCollection js)
        {
            return CommonGestureDecisions.StretchedOutFully(js, JointID.HandLeft) &&
                   CommonGestureDecisions.StretchedOutFully(js, JointID.HandRight);
        }
    }
}
