﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;
using System.Windows;
using EventGeneration;
using sensor;
using Sensor.Gesture.Transform;
using Sensor.MouseAction;
using Sensor.Common;

namespace Sensor.Gesture.GestureHandler
{
    public class TwoHandsMoveTogetherGesture : IGestureHandler
    {
        private ITransform m_transform;
        private Point? m_previous00;
        private IStatus m_status;

        private MouseMoveAction m_mouse_movement;

        public TwoHandsMoveTogetherGesture(ITransform transform, IStatus status)
        {
            m_transform = transform;
            m_previous00 = null;
            m_status = status;
            m_mouse_movement = new MouseMoveAction(transform);
        }

        public bool HandleMoveEvent(JointsCollection js)
        {
            if (!KeepHandlingEvent(js))
            {
                EventGeneration.MouseEvent.EventEndRightDrag();
                EventGeneration.KeyboardEvent.EventKeyUpShift();
                return false;
            }

            m_status.SetStatus1("TwoHandsMoveTogetherGesture");
            if (!m_previous00.HasValue)
            {
                m_previous00 = JointUtil.RightHand(js);
                EventGeneration.KeyboardEvent.EventKeyDownShift();
                EventGeneration.MouseEvent.EventStartRightDrag();
            }

            m_mouse_movement.MoveBy(JointUtil.RightHand(js));
            return true;
        }

        public bool KeepHandlingEvent(JointsCollection js)
        {
            return CommonGestureDecisions.StretchedOutPartially(js, JointID.HandLeft) ||
                   CommonGestureDecisions.StretchedOutPartially(js, JointID.HandRight);
        }

        public bool CanHandleEvent(JointsCollection js)
        {
            return CommonGestureDecisions.StretchedOutFully(js, JointID.HandLeft) && 
                   CommonGestureDecisions.StretchedOutFully(js, JointID.HandRight);
        }
    }
}
