﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Forms;
using Microsoft.Research.Kinect.Nui;
using Sensor.Gesture.Common;

namespace Sensor.Gesture.Transform
{
    public class Transform : ITransform
    {
        private double ScreenWidth = Screen.PrimaryScreen.Bounds.Width;
        private double ScreenHeight = Screen.PrimaryScreen.Bounds.Height;
        private double m_control_square_dimension_parameter_space;

        public Transform()
        {
            m_control_square_dimension_parameter_space = 1;
            Main.NUI.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(EventSkeletonFrameReady);
        }

        public void EventSkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            SkeletonData skeleton = 
                (from s in e.SkeletonFrame.Skeletons 
                 where s.TrackingState == SkeletonTrackingState.Tracked 
                 select s
                ).FirstOrDefault();
            if (skeleton == null) return;
            Point shoulder_center_parameter_space = GetJointPosition(skeleton.Joints[JointID.ShoulderCenter]);
            Point hip_center_parameter_space = GetJointPosition(skeleton.Joints[JointID.HipCenter]);
            m_control_square_dimension_parameter_space = 
                PointUtil.Distance(shoulder_center_parameter_space, hip_center_parameter_space);
        }

        public Point ParameterSpaceToScreenSpace(Point point)
        {
            return new Point(XDistanceToScreenDistance(point.X), YDistanceToScreenDistance(point.Y));
        }

        public double XDistanceToScreenDistance(double x_distance)
        {
            return x_distance * 5000;
        }

        public double YDistanceToScreenDistance(double y_distance)
        {
            return y_distance * 5000;
        }

        private Point GetJointPosition(Joint joint)
        {
            float depth_x, depth_y;
            Main.NUI.SkeletonEngine.SkeletonToDepthImage(joint.Position, out depth_x, out depth_y); // [0.0 1.0]
            return new Point(depth_x, depth_y);
        }

    }
}
