﻿using System;
using Microsoft.Research.Kinect.Nui;
using sensor;
using Sensor.Gesture;

namespace Sensor
{
    public class Main
    {
        public static Runtime NUI { get; private set; }

        public Main(IStatus status) {
            NUI = new Runtime();
            
            try {
                NUI.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking | RuntimeOptions.UseColor);
            }catch (InvalidOperationException) {
                throw new Exception("Runtime initialization failed. Please make sure Kinect device is plugged in.");
            }

            try {
                NUI.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
                NUI.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            } catch (InvalidOperationException) {
                throw new Exception("Failed to open stream. Please make sure to specify a supported image type and resolution.");
            }

            NUI.SkeletonEngine.TransformSmooth = true;
            var parameters = new TransformSmoothParameters {
                Smoothing = 0.75f,
                Correction = 0.0f,
                Prediction = 0.0f,
                JitterRadius = 0.05f,
                MaxDeviationRadius = 0.01f
            };
            NUI.SkeletonEngine.SmoothParameters = parameters;

            IGestureDispatcher gesture_dispatcher = new GestureDispatcher(status);
            gesture_dispatcher.Start();
        }

        public void Finish()
        {
            NUI.Uninitialize();
        }
    }
}
