﻿using Microsoft.Research.Kinect.Nui;
using System;
using System.Windows;
using Sensor.Gesture.Transform;

namespace Sensor.MouseAction
{
    public class GenericMouseMoveAction : IMouseMoveAction
    {
        private MouseMoveAction m_mouse_movement_action;
        private MouseButton? m_button;
        private Key? m_key;

        private bool m_first_time;

        public GenericMouseMoveAction(ITransform transform, MouseButton button)
               :this(transform, button, null){}

        public GenericMouseMoveAction(ITransform transform, Key key)
               :this(transform, null, key){}

        public GenericMouseMoveAction(ITransform transform, MouseButton? button, Key? key)
        {
            m_mouse_movement_action = new MouseMoveAction(transform, 40);
            m_button = button;
            m_key = key;
            m_first_time = true;
        }

        public void MoveBy(Point new_position)
        {
            if (m_first_time)
            {
                StartDrag(m_button);
                HoldKeyDown(m_key);
                m_first_time = false;
            }
            m_mouse_movement_action.MoveBy(new_position);
        }

        public void Finish()
        {
            EndDrag(m_button);
            ReleaseKey(m_key);
            m_mouse_movement_action.Finish();
            m_first_time = true;
        }

        private static void StartDrag(MouseButton? button)
        {
            if (!button.HasValue) return;
            switch (button)
            {
                case MouseButton.Left:
                    EventGeneration.MouseEvent.EventStartLeftDrag();
                    break;
                case MouseButton.Right:
                    EventGeneration.MouseEvent.EventStartRightDrag();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + button);
            }
        }

        private static void EndDrag(MouseButton? button)
        {
            if (!button.HasValue) return;
            switch (button)
            {
                case MouseButton.Left:
                    EventGeneration.MouseEvent.EventEndLeftDrag();
                    break;
                case MouseButton.Right:
                    EventGeneration.MouseEvent.EventEndRightDrag();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + button);
            }
        }

        private static void HoldKeyDown(Key? key)
        {
            if (!key.HasValue) return;
            switch (key)
            {
                case Key.Shift:
                    EventGeneration.KeyboardEvent.EventKeyDownShift();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + key);
            }
        }

        private static void ReleaseKey(Key? key)
        {
            if (!key.HasValue) return;
            switch (key)
            {
                case Key.Shift:
                    EventGeneration.KeyboardEvent.EventKeyUpShift();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + key);
            }
        }
    }
}
