﻿using Microsoft.Research.Kinect.Nui;
using System;
using System.Windows;
using Sensor.Gesture.Transform;

namespace Sensor.MouseAction
{
    public class MouseDragAction : IMouseMoveAction
    {
        private MouseMoveAction m_mouse_movement_action;
        private MouseButton m_button;
        private bool m_first_time;

        public MouseDragAction(ITransform transform, MouseButton button)
        {
            m_mouse_movement_action = new MouseMoveAction(transform, 40);
            m_button = button;
            m_first_time = true;
        }

        public void MoveBy(Point new_position)
        {
            if (m_first_time)
            {
                StartDrag();
            }
            m_mouse_movement_action.MoveBy(new_position);
        }

        public void Finish()
        {
            EndDrag();
            m_mouse_movement_action.Finish();
            m_first_time = true;
        }

        private void StartDrag()
        {
            switch (m_button)
            {
                case MouseButton.Left:
                    EventGeneration.MouseEvent.EventStartLeftDrag();
                    break;
                case MouseButton.Right:
                    EventGeneration.MouseEvent.EventStartRightDrag();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + m_button);
            }
        }

        private void EndDrag()
        {
            switch (m_button)
            {
                case MouseButton.Left:
                    EventGeneration.MouseEvent.EventEndLeftDrag();
                    break;
                case MouseButton.Right:
                    EventGeneration.MouseEvent.EventEndRightDrag();
                    break;
                default:
                    throw new ArgumentException("Unexpected case : " + m_button);
            }
        }
    }
}
