﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Sensor.Gesture.Common;
using EventGeneration;
using Sensor.Gesture.Transform;

namespace Sensor.MouseAction
{
    public class MouseMoveAction : IMouseMoveAction
    {
        private ITransform m_transform;
        private Queue<Point> m_positions_so_far;
        private static int s_default_points_to_remember = 20;
        private int m_points_to_remember;

        public MouseMoveAction(ITransform transform):
            this(transform, s_default_points_to_remember){}

        public MouseMoveAction(ITransform transform, int points_to_remember)
        {
            m_transform = transform;
            m_positions_so_far = new Queue<Point>();
            m_points_to_remember = points_to_remember;
        }

        public void MoveBy(Point new_position)
        {
            m_positions_so_far.Enqueue(new_position);
            if (m_positions_so_far.Count < m_points_to_remember)
            {
                return;
            }
            
            m_positions_so_far.Dequeue();

            Point difference_sum = new Point(0, 0);
            Point previous = m_positions_so_far.First();
            foreach(var p in m_positions_so_far){
                difference_sum = PointUtil.Add(difference_sum, PointUtil.Subtract(p, previous));
                previous = p;
            }
            Point difference_average = new Point(difference_sum.X / (m_positions_so_far.Count - 1), difference_sum.Y / (m_positions_so_far.Count - 1));
            EventGeneration.MouseEvent.EventMoveMouse(m_transform.ParameterSpaceToScreenSpace(difference_average));
        }

        public void Finish()
        {
            m_positions_so_far.Clear();
        }
    }
}

