﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Sensor.Gesture.Common;
using EventGeneration;
using sensor;
using Sensor.Gesture.Transform;

namespace Sensor.MouseAction
{

    public interface ISmoothedTwoPointMouseMovementAction
    {
        void MoveTo(Point left_hand, Point right_hand);
        void Reset();
    }

    public class TwoPointMouseMoveAction : ISmoothedTwoPointMouseMovementAction
    {
        private ITransform m_transform;
        private Queue<Tuple<Point, Point>> m_positions_so_far;
        private static int s_default_points_to_remember = 20;
        private int m_points_to_remember;
        private IStatus m_status;

        public TwoPointMouseMoveAction(ITransform transform, IStatus status):this(transform, s_default_points_to_remember, status){}

        public TwoPointMouseMoveAction(ITransform transform, int points_to_remember, IStatus status)
        {
            m_transform = transform;
            m_positions_so_far = new Queue<Tuple<Point, Point>>();
            m_points_to_remember = points_to_remember;
            m_status = status;
        }

        public void MoveTo(Point new_left_position, Point new_right_position)
        {
            m_positions_so_far.Enqueue(new Tuple<Point, Point>(new_left_position, new_right_position));
            if (m_positions_so_far.Count < m_points_to_remember)
            {
                return;
            }
            
            m_positions_so_far.Dequeue();

            double difference_sum = 0;
            Tuple<Point, Point> previous = m_positions_so_far.First();
            foreach (var pair in m_positions_so_far)
            {
                double current_difference = PointUtil.Distance(pair.Item2, pair.Item1);
                double previous_difference = PointUtil.Distance(previous.Item2, previous.Item1);
                difference_sum += current_difference - previous_difference;
                previous = pair;
            }
            double averaged_difference = difference_sum / m_positions_so_far.Count;
            Point up_down_movememnt = new Point(0, -1 * averaged_difference / 5); // -1 ==> for some reason
            Point up_down_movement_screen_space = m_transform.ParameterSpaceToScreenSpace(up_down_movememnt);
            EventGeneration.MouseEvent.EventMoveMouse(up_down_movement_screen_space);
            m_status.SetStatus1("TwoHandMovementAction. Moving.");
        }

        public void Reset()
        {
            m_positions_so_far.Clear();
        }
    }
}

